#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um sich fr Spiele anzumelden
    //  Kann nur von eingeloggten Clients verwendet werden
    class AnmeldungClient : public AnmeldungServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        AnmeldungClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~AnmeldungClient();
        // verbindet sich mit dem zugewiesenen Anmeldung Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // erstellt eine gruppe
        //  karteId: Die Id der Karte fr die die Gruppe erstellt werden soll
        //  Gibt bei Erfolg die Id der Gruppe zurck. Bei misserfolg wird 0 zurckgegeben
        int gruppeErstellen( int karteId ) override;
        // Betritt eine Gruppe
        //  gruppeId: Die Id der Gruppe
        //  mitglieder: Nach erfolgreichem Aufruf enthlt diese Liste die Lister der Account Ids der Gruppenmitglieder
        //  anzahl: Nach erfolgreichem Aufruf wird hier die Anzahl der Gruppenmitglieder gespeichert
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeBetreten( int gruppeId, Framework::Array< int > *mitglieder, int *anzahl ) override;
        // Verlsst eine Gruppe
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeVerlassen( int gruppeId ) override;
        // Trgt die Gruppe in die Warteschlange fr die Karte ein (nur fr den Gruppen Administrator)
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeAnmelden( int gruppeId ) override;
        // Trgt die Gruppe aus der Warteschlange fr die Karte aus (nur fr den Gruppen Administrator)
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeAbmelden( int gruppeId ) override;
        // lht Spieler ein die Gruppe zu betreten (nur fr den Gruppen Administrator)
        //  accountId: Die Id des Accounts
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeSpielerEinladen( int accountId, int gruppeId ) override;
        // Zieht die Einladung eine Gruppe zu betreten zurck (nur fr den Gruppen Administrator)
        //  accountId: Die Id des Accounts
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeEinladungAbbrechen( int accountId, int gruppeId ) override;
        // Einladung zur Gruppe ablehnen
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeEinladungAblehnen( int gruppeId ) override;
        // wirft Spieler aus Gruppe (nur fr den Gruppen Administrator)
        //  accountId: Die Id des Accounts
        //  gruppeId: Die Id der Gruppe
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool kickSpielerAusGruppe( int accountId, int gruppeId ) override;
        // Legt fest, ob die Spieler in der Gruppe alleine spielen wollen, oder ob weitere Spieler aus der Warteschlange hinzugefgt werden sollen bis die Karte voll ist
        //  gruppeId: Die Id der Gruppe
        //  spielStarte: 1:=alleine spielen, 0:=auf volles Spiel warten
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeSpielStarten( int gruppeId, bool spielStarten ) override;
        // Sendet eine Nachricht an den Gruppen Chat
        //  gruppeId: Die Id der Gruppe
        //  nachricht: Die Nachricht, die gesendet werden soll
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool gruppeNachricht( int gruppeId, char *nachricht ) override;
        // Meldet den eingeloggten Account in der Warteschlange fr eine Karte an
        //  karteId: Die Id der Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool anmelden( int karteId ) override;
        // Meldet den eingeloggten Account aus der Warteschlange ab
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool abmelden() override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        AnmeldungServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        AnmeldungServerClient *release() override;
    };
}