#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um Chat nachrichten zu senden oder zu empfangen. Das Serversystem sendet auerdem einiege Mitteilungen ber den Chat (z.B. Gefundene Spiele, Gruppeneinladungen ...)
    //  Kann nur von eingeloggten Clients verwendet werden
    class ChatClient : public ChatServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *senden;
        Network::Klient *empfangen;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        ChatClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~ChatClient();
        // verbindet sich mit dem zugewiesenen Chat Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // sendet eine Chat Nachricht zu einem Account
        //  zuAccount: Id des Ziel Accounts
        //  nachricht: Die Nachricht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatNachricht( int zuAccount, const char *nachricht ) override;
        // ndert den angezeigten Account Namen
        //  name: der neue Name
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool accountNamendern( const char *name ) override;
        // beendet die Freundschaft mit einem Account
        //  accountId: Die Id des Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool freundschaftBeenden( int accountId ) override;
        // sendet eine Freundesanfragezu einem Account
        //  accountId: Die Id des Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool freundesAnfrage( int accountId ) override;
        // beantwortet eine Freundesanfrage
        //  accountId: Die Id des Accounts
        //  ja: 1:=annehmen,0:=ablehnen
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool freundesAnfrageBeantworten( int accountId, bool ja ) override;
        // erstellt ein chatroom
        //  name: Der Name des Chatrooms
        //  Gibt bei erfolg die Id des Chatrooms zurck, 0 sonst
        int chatroomErstellen( const char *name ) override;
        // Sendet eine Einladung zum Chatroom
        //  accountId: Die Id des Accounts der eingeladen werden soll
        //  chatroomId: Die Id des Chatrooms
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomEinladung( int accountId, int chatroomId ) override;
        // Einladung zum Chatroom wird abgelehnt
        //  accountId: Die Id des Accounts
        //  chatroomId: Die Id des Chatrooms
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomEinladungAblehnen( int accountId, int chatroomId ) override;
        // betritt ein chatroom
        //  chatroomId: Die Id des Chatrooms
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomBetreten( int chatroomId ) override;
        // chatroom Nachricht senden
        //  chatroomId: Die Id des Chatrooms
        //  nachricht: Die Nachricht die gesendet werden soll
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomNachricht( int chatroomId, const char *nachricht ) override;
        // verlsst chatroom
        //  chatroomId: Die Id des Chatrooms
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomVerlassen( int chatroomId ) override;
        // kickt Account aus Chatroom (nur wenn als admin des Chatrooms eingeloggt)
        //  chatroomId: Die Id des Chatrooms
        //  accountId: Die Id des Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatroomKick( int chatroomId, int accountId ) override;
        // fragt nach allen freunden
        //  Die Liste der Freunde kann ber getNextMessage empfangen werden
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool freundesListeAnfragen() override;
        // fragt nach allen chatnachrichten, die gesendet wurden, whrend der Account in den der Client eingeloggt ist, offline war
        //  Die einzelnen Nachrichten knnen ber getNextMessage empfangen werden
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool chatNachrichtAnfrage() override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Wartet auf eine Nachricht vom Chatserver.
        //  Gibt bei Erfolg 1 zurck, 0 sonnst
        //  nachricht: Eine Referenz auf die Struktur, in der die Nachricht gespeichert werden soll
        bool getNextMessage( ChatServerNachricht &nachricht ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        ChatServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        ChatServerClient *release() override;
    };
}