#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um Karten fr Spiele zu erstellen
    //  Kann nur von eingeloggten Clients verwendet werden
    class EditorClient : public EditorServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        EditorClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~EditorClient();
        // verbindet sich mit dem zugewiesenen Editor Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // Erstellt eine neue Karte
        //  name: Der Name der Karte
        //  spielArt: Die Id des Spiels
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool karteErstellen( char *name, int spielArt ) override;
        // ldt eine bestimmte Karte
        //  id: Die Id der Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeKarte( int id ) override;
        // gibt eine Liste mit Abbildern der geladenen Karte zurck
        //  name: Enthlt nach erfolgreichem Aufruf eine Liste mit Namen der Abbilder
        //  datum: Enthlt nach erfolgreichem Aufruf eine Liste mit dem Datum der Erstellung fr jedes Abbild
        //  Gibt die Anzahl der Abbilder zurck
        int getAbbildListe( Framework::RCArray< Framework::Text > *name, Framework::RCArray< Framework::Zeit > *datum ) override;
        // Erstellt ein neues Abbild der geladenen Karte
        //  name: Der Name des Abbildes
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool abbildErstellen( char *name ) override;
        // Lscht ein altes Abbild der geladenen Karte
        //  name: Der Name des Abbildes
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool abbildLschen( char *name ) override;
        // Stellt ein altes Abbild der geladenen Karte wieder her
        //  name: Der Name des Abbildes
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool abbildHerstellen( char *name ) override;
        // Verffentlich eine neue Version der geladenen Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verffentlichen() override;
        // Verffentlich eine neue Version der Shop Seite der geladenen Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool shopSeiteVerffentlichen() override;
        // Lht die Vorschau der Shop Seite der geladenen Karte herrunter und speichert sie unter data/tmp/ke/ssv
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeShopSeiteVorschau() override;
        // Erstellt eine neue Shop Seiten Datei
        //  name: Der Name der Datei
        //  typ: 0=Ordner, 1=Bild, 2=Text
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssDateiErstellen( char *name, int typ ) override;
        // ffnet einen Ordner der Shop Seite
        //  name: Der Name des Ordners
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssOrdnerffnen( char *name ) override;
        // Ldt das ein Bild der Shop Seite und gibt es zurck
        //  datei: Der Name der Bilddatei
        //  bild: Der Name des Bildes
        //  Gibt bei misserfolg 0 zurck
        Framework::Bild *ssBildLaden( char *datei, char *bild ) override;
        // Lscht eine Datei der Shop Seite
        //  name: Der Name der Datei
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssDateiLschen( char *name ) override;
        // Ld eine KSGS Datei der Shop Seite und gibt den Quellcode zurck
        //  datei: Der Name der Datei
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *ssTextLaden( char *datei ) override;
        // Spechert eine KSGS Datei der Shop Seite
        //  datei: Der Name der Datei
        //  zText: Ein Zeiger auf den neuen Quellcode (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssTextSpeichern( char *datei, Framework::Text *zText ) override;
        // Lscht ein Bild der Shop Seite
        //  datei: Der Name der Bilddatei
        //  bild: Der Name des Bildes
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssBildLschen( char *datei, char *bild ) override;
        // Speichert ein Bild einer ShopSeite
        //  datei: Der name der Bilddatei
        //  name: Der Name des Bildes
        //  zBild: Ein Zeiger auf das Bild (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ssBildSpeichern( char *datei, char *name, Framework::Bild *zBild ) override;
        // Ldt eine Liste mit Dateien von der Shop Seite
        //  zList: Enthlt nach erfolgreichem Aufruf die Liste mit Dateinamen
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        int ssGetDateiListe( Framework::RCArray< Framework::Text > *zList ) override;
        // Ldt die Bild Liste einer Bild Datei der Shop Seite
        //  name: Der Name der Bilddatei
        //  zList: Enthlt nach erfolgreichem Aufruf eine Liste mit Bildnamen
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        int ssGetBildListe( char *name, Framework::RCArray< Framework::Text > *zList ) override;
        // Ldt die Shop Daten
        //  es: Enthlt nach erfolgreichem aufruf den Erwerbbarkeits Status (0 = nichts, 1 = nur Testversion, 2 = nur Vollversion, 3 = beides)
        //  tp: Enthlt nach erfolgreichem Aufruf den Preis der Testversion in Kupfer
        //  vp: Enthlt nach erfolgreichem Aufruf den Preis der Vollversion in Kupfer
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool getShopDaten( int &es, int &tp, int &vp ) override;
        // Setzt die Shop Daten
        //  es: der Erwerbbarkeits Status (0 = nichts, 1 = nur Testversion, 2 = nur Vollversion, 3 = beides)
        //  tp: der Preis der Testversion in Kupfer
        //  vp: der Preis der Vollversion in Kupfer
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool setShopDaten( int es, int tp, int vp ) override;
        // gibt die Kartenbeschreibung als KSGScript Quellcode zurck
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *beschreibungLaden() override;
        // gibt das Titelbild der Karte zurck
        //  Gibt bei misserfolg 0 zurck
        Framework::Bild *titelbildLaden() override;
        // gibt das Minimap Bild der Karte zurck
        //  Gibt bei misserfolg 0 zurck
        Framework::Bild *minimapLaden() override;
        // gibt das Ladebild der Karte zurck
        //  Gibt bei misserfolg 0 zurck
        Framework::Bild *ladebildLaden() override;
        // speichert den KSGScript Quellcode der Kartenbeschreibung
        //  zText: Der KSGScript Quellcode (ohne erhhten reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool beschreibungSpeichern( Framework::Text *zText ) override;
        // speichert das Titelbild der Karte
        //  zBild: das neue Titelbild
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool titelbildSpeichern( Framework::Bild *zBild ) override;
        // speichert das Minimapbild der Karte
        //  zBild: das neue Minimapbild
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool minimapSpeichern( Framework::Bild *zBild ) override;
        // speichert das Ladebild der Karte
        //  zBild: das neue Ladebild
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladebildSpeichern( Framework::Bild *zBild ) override;
        // ldt die Spieler Team Daten der Karte
        //  sts: Enthlt nach erfolgreichem Aufruf die Spieler Team Daten
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeTeamDaten( SpielerTeamStrukturV *sts ) override;
        // speichert die Team Daten
        //  sts: die neuen Spieler Team Daten
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool speicherTeamDaten( SpielerTeamStrukturV *sts ) override;
        // Erstellt neue Datei im Datei Editor
        //  typ: 0=ordner 1=bild 2=modell2d 3=sound
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deNeueDatei( char typ, Framework::Text *zName ) override;
        // ffnet Ordner im Datei Editor
        //  zName: Der Name des Ordners
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deOrdnerffnen( Framework::Text *zName ) override;
        // Gibt ein Bild aus dem Datei Editor zurck
        //  zDatei: Der Name der Datei (ohne erhhten Reference Counter)
        //  zBild: Der Name des Bildes (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt bei misserfolg 0 zurck
        Framework::Bild *deBildLaden( Framework::Text *zDatei, Framework::Text *zBild, Framework::FBalken *zF ) override;
        // Lscht eine Datei aus dem Datei Editor
        //  zName: Der Name der Datei
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deDateiLschen( Framework::Text *zName ) override;
        // Lscht Bild aus Datei im Datei Editor
        //  zDatei: Der Name der Bilddatei (ohne erhhten Reference Counter)
        //  zBild: Der Name des Bildes (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deBildLschen( Framework::Text *zDatei, Framework::Text *zBild ) override;
        // Speichert Bild in Datei im Datei Editor
        //  zDatei: Der Name der Bilddatei (ohne erhhten Reference Counter)
        //  zName: Der Name des Bildes (ohne erhhten Reference Counter)
        //  zBild: Das neue Bild (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deBildSpeichern( Framework::Text *zDatei, Framework::Text *zName, Framework::Bild *zBild, Framework::FBalken *zF ) override;
        // Ldt eine Liste mit Dateien im aktuellen Ordner des Datei Editors
        //  zNamen: Enthlt nach erfolgreichem Aufruf eine Liste mit Dateinamen (ohne erhhten Reference Counter)
        //  Gibt die Anzahl der Dateien zurck
        int deGetDateiListe( Framework::RCArray< Framework::Text > *zNamen ) override;
        // Ldt eine Liste mit Bildern aus einer Bilddatei des Datei Editors
        //  zNamen: Enthlt nach erfolgreichem Aufruf eine Liste mit den Bildnamen (ohne erhhten Reference Counter)
        //  Gibt die Anzahl der Bilder zurck
        int deGetBildListe( Framework::Text *zDatei, Framework::RCArray< Framework::Text > *zNamen ) override;
        // Ldt eine Liste mit 2D Modellen aus einer Modeldatei im Datei Editor
        //  zDatei: Der Name der Modeldatei (ohne erhhten Reference Counter)
        //  zNamen: Enthlt nach erfolgreichem Aufruf eine Liste mit den Modelnamen (ohne erhhten Reference Counter)
        //  Gibt die Anzahl der Modelle zurck
        int deGetModelListe( Framework::Text *zDatei, Framework::RCArray< Framework::Text > *zNamen ) override;
        // Ldt eine Liste mit Sounds aus einer Sounddatei im Datei Editor
        //  zDatei: Der Name der Sounddatei (ohne erhhten Reference Counter)
        //  zNamen: Enthlt nach erfolgreichem Aufruf eine Liste mit den Soundnamen (ohne erhhten Reference Counter)
        //  Gibt die Anzahl der Sounds zurck
        int deGetSoundListe( Framework::Text *zDatei, Framework::RCArray<  Framework::Text > *zNamen ) override;
        // Lscht ein Model aus einer Modeldatei des Datei Editors
        //  zDatei: Der Name der Modeldatei (ohne erhhten Reference Counter)
        //  zModel: Der Name des Models (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deModelLschen( Framework::Text *zDatei, Framework::Text *zModel ) override;
        // Lscht einen Sound aus einer Sounddatei des Datei Editors
        //  zDatei: Der Name der Sounddatei (ohne erhhten Reference Counter)
        //  zSound: Der Name des Sounds (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deSoundLschen( Framework::Text *zDatei, Framework::Text *zSound ) override;
        // Ldt ein Model aus einer Modeldatei des Datei Editors
        //  zDatei: Der Name der Modeldatei (ohne erhhten Reference Counter)
        //  zModel: Enthlt nach erfolgreichem Aufruf das Model (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        Framework::Model2DData *deModelLaden( Framework::Text *zDatei, Framework::Text *zModel, Framework::FBalken *zF ) override;
        // Ldt einen Sound herunter und gibt ihn zurck
        //  file: Ein Zeiger auf die GSLDatei, in der der Sound gespeichert werden soll
        //  zDatei: Der Name der Sounddatei (ohne erhhten Reference Counter)
        //  zSound: Der Name des Sounds (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt bei misserfolg 0 zurck
        GSL::GSLSoundV *deSoundLaden( GSL::GSLDateiV *file, Framework::Text *zDatei, Framework::Text *zSound, Framework::FBalken *zF ) override;
        // Speichert ein Model in eine Modeldatei des Datei Editors
        //  zDatei: Der Name der Modeldatei (ohne erhhten Reference Counter)
        //  zModel: Der Name des Models (ohne erhhten Reference Counter)
        //  zData: Die Daten des Models (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deModelSpeichern( Framework::Text *zDatei, Framework::Text *zModel, Framework::Model2DData *zData, Framework::FBalken *zF ) override;
        // Speichert einen Sound in einer Sounddatei des Datei Editors
        //  zDatei: Der Name der Modeldatei (ohne erhhten Reference Counter)
        //  zSound: Der Name des Sounds (ohne erhhten Reference Counter)
        //  zData: Die Daten des Sounts (ohne erhhten Reference Counter)
        //  zF: Ein Fortschrittsbalken, der automatisch aktualisiert wird (ohne erhhten Reference Counter)
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool deSoundSpeichern( Framework::Text *zDatei, Framework::Text *zSound, GSL::GSLSoundV *zData, Framework::FBalken *zF ) override;
        // Ldt die Kauf Statistik der geladenen Karte
        //  verkauft: Enthlt nach erfolgreichem Aurfuf eine Liste mit den Anzahlen der Kufe der Karte der letzten 30 Tage
        //  einkommen: Enthlt nach erfolgreichem Aurfuf eine Liste mit den Anzahlen der verdinten Kupfer in den letzten 30 Tagen
        //  gespielt: Enthlt nach erfolgreichem Aurfuf eine Liste mit den Anzahlen der Spielen der letzten 30 Tage
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        int getVerkaufStatistik( Framework::Array< int > *verkauft, Framework::Array< int > *einkommen, Framework::Array< int > *gespielt ) override;
        // Startet den Editor Modus, welcher fr jedes Spiel unterschiedlich funktioniert
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool initEditor() override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        EditorServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        EditorServerClient *release() override;
    };
}