#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um informationen ber Accounts abzufragen
    //  Kann nur von eingeloggten Clients verwendet werden
    class InformationClient : public InformationServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        InformationClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~InformationClient();
        // verbindet ich mit dem zugewiesenen Informaion Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // fragt nach dem Informationstext
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  txt: Ein Zeiger auf ein Text Objekt, welches bei einem erfolgreichen Aufruf den Informationstext enthlt
        //  typ: ein Zeieger auf ein Integer, welches bei einem erfolgreichen Aufruf den Typ der Information enthlt
        bool getInformationText( Framework::Text *txt, int *typ ) override;
        // fragt, ob das entsprechende Spiel vom Server gesperrt wurde
        //  spielId: Die Id des Spiels
        //  Gibt 1 zurck, falls das Spiel gespielt werden darf, 0 sonnst
        bool istSpielErlaubt( int spielId ) override;
        // fragt, ob die entsprechende Karte vom Server gesperrt wurde
        //  karteId: Die Id der Karte
        //  Gibt 1 zurck, falls die Karte gespielt werden darf, 0 sonnst
        bool istKarteErlaubt( int karteId ) override;
        // Ermittelt die Id einer Karte
        //  name: Der Name der Karte
        //  Gibt bei misserfolg 0 zurck
        int getKarteId( char *name ) override;
        // Fragt nach der Id eines Spiels
        //  name: Der Name des Spiels
        //  Gibt bei misserfolg 0 zurck
        int getSpielId( char *name ) override;
        // fragt nach dem Namen eines Accounts
        //  accountId: Die Id des Accounts
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *getSpielerName( int accountId ) override;
        // fragt nach der Spiel Statistik eines Accounts
        //  accountId: Die Id des Accounts
        //  spielId: Die Id des Spiels
        //  werte: Nach erfolgreichem Aufruf enthlt die Liste die Statistik werte (gespielt, gewonnen, verloren, punkte, bw0, bw1, bw2, bw3, bw4, bw5)
        bool getSpielStatistik( int accountId, int spielId, Framework::Array< int > *werte ) override;
        // fragt nach dem Namen einer Karte
        //  karteId: Die id der Karte
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *getKarteName( int karteId ) override;
        // fragt nach dem Namen eines Spiels
        //  spielId: Die id des Spiels
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *getSpielName( int spielId ) override;
        // fragt nach dem Namen eines Chatrooms
        //  chatroomId: Die id des Chatrooms
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *getChatroomName( int chatroomId ) override;
        // fragt zu welcher Spielart die Karte gehrt
        //  karteId: Die id der Karte
        //  Gibt bei misserfolg 0 zurck
        int getSpielId( int karteId ) override;
        // gibt die Id eines Accounts zurck
        //  name: Der Angezeigte Name des Accounts
        //  Gibt bei misserfolg 0 zurck
        int getAccountId( char *name ) override;
        // gibt die Id des Chatrooms zurck
        //  name: Der name des Chatrooms
        //  Gibt bei misserfolg 0 zurck
        int getChatroomId( char *name ) override;
        // gibt die karten Id zurck, zu dem eine Gruppe erstellt wurde
        //  gruppenId: Die id der Gruppe
        //  Gibt bei misserfolg 0 zurck
        int getGruppenKarteId( int gruppenId ) override;
        // gibt 1 zurck, wenn zu einer Gruppe Spieler hinzugefgt werden knnen, 0 sonnst
        //  gruppeId: Die id der Gruppe
        bool getGruppeSpielerHinzufgen( int gruppeId ) override;
        // gibt die Account Id des Gruppen Administrators zurck
        //  gruppeId: Die Id der Gruppe
        //  Gibt bei misserfolg 0 zurck
        int getGruppeAdminId( int gruppeId ) override;
        // gibt die Punkte eines Spielers zurck
        //  accountId: Die Accont Id des Spielers
        //  spielId: Die Id des Spiels
        //  Gibt bei misserfolg 0 zurck
        int getSpielerPunkte( int accountId, int spielId ) override;
        // gibt eine Liste mit Ids von gekauften Spielen zurck
        //  Gibt bei misserfolg 0 zurck
        Framework::Array< int > *getAccountSpielArtListe() override;
        // gibt die neuste Version eines Spiels zurck
        //  spielId: Die Id des Spiels
        //  Gibt bei misserfolg 0 zurck
        int getSpielVersion( int spielId ) override;
        // gibt das Kupfer des Accounts zurck
        //  Gibt bei misserfolg 0 zurck
        int getKupfer() override;
        // Gibt die Dateigruppem Id eines Spieles zurck
        //  spielId: Die Id des Spiels
        //  Gibt bei misserfolg 0 zurck
        int getDateiGruppeIdVonSpiel( int spielId ) override;
        // Gibt den Dateigruppen Pfad zurck
        //  dgId: Die Id der Dateigruppe
        //  Gibt bei misserfolg 0 zurck
        Framework::Text *getDateiGruppePfad( int dgId ) override;
        // gibt eine Liste mit gekauften Karten zurck
        //  spielId: Die Spiel Id zu der die Karten gehren sollen
        //  Gibt bei misserfolg 0 zurck
        Framework::Array< int > *getAccountKarteListe( int spielId ) override;
        // Gibt die Dateigruppen Id eines Pfades zurck
        //  pfad: Der Pfad der Dateigruppe
        //  Gibt bei misserfolg 0 zurck
        int getDateiGruppeIdVonPfad( char *pfad ) override;
        // gibt die neuste Version einer Dateigruppe zurck
        //  dg: Die Dateigruppen Id
        //  Gibt bei misserfolg 0 zurck
        int getDateiGruppeVersion( int dg ) override;
        // Gibt eine Liste Mit Accounts zurck, die einen bestimmten Text im Namen haben und bestimmt Sortiert ist
        //  suche: Der Text, der in den Namen enthalten sein soll
        //  seite: Die gewnschte Seite der Tabelle (enthlt nach einem Erfolgreichen Aufruf die korrekte Seite)
        //  maxSeite: Enthlt nach erfolgreichem Aufruf die Maximale Seite
        //  sortSpalte: Die Spalte, nach der Sortiert werden soll
        //  rckwrts: 1:=Rckwrtzt Sortierung, 0:=Vorwrts Sortierung
        //  accounts: Enthlt nach erfolgreichem Aufruf die Daten der Accounts
        //  Gibt die Anzahl der gefundenen Accounts zurck
        //  Gibt die Anzahl der Spieler zurck, -1 bei fehler
        int getSpielerListe( char *suche, int *seite, int *maxSeite, char sortSpalte, char rckwrts, Framework::Array< AccountInfo > *accounts ) override;
        // ermittelt die aktivitt eines Spielers
        //  accId: Die Id des Accounts
        //  info: Enthlt nach erfolgreichem Aufruf die Aktivittsinformationen der letzten 30 Tage
        bool getSpielerAktivitt( int accId, Framework::Array< AccountActivityInfo > *info ) override;
        // Gibt zurck, wie oft ein Spieler eine Karte schon gespielt hat
        //  account: Die Id des Accounts
        //  karte: Die Id der Karte
        int getAccountKarteSpiele( int account, int karte ) override;
        // Gibt zurck, wie oft ein Spieler eine Karte schon gewonnen hat
        //  account: Die Id des Accounts
        //  karte: Die Id der Karte
        int getAccountKarteSpieleGewonnen( int account, int karte ) override;
        // Gibt zurck, ob ein Spieler die Karte im Besitz hat
        //  account: Die Id des Accounts
        //  karte: Die Id der Karte
        bool hatAccountKarte( int account, int karte ) override;
        // Gibt zurck, ob ein Spieler ein Spiel im Besitz hat
        //  account: Die Id des Accounts
        //  spiel: Die Id des Spiels
        bool hatAccountSpiel( int account, int spiel ) override;
        // gibt eine Liste mit Ids von Karten zurck, die von einem Spieler bereits gespielt wurden
        //  account: Die Id des Accounts
        //  spiel: Die Id des Spiels 
        //  Gibt bei misserfolg 0 zurck
        Framework::Array< int > *getAccountKarteGespieltListe( int account, int spielId ) override;
        // gibt eine Liste mit Ids von Spielen zurck, die von einem Spieler bereits gespielt wurden
        //  account: Die Id des Accounts
        //  Gibt bei misserfolg 0 zurck
        Framework::Array< int > *getAccountSpielGespieltListe( int account ) override;
        // Gibt eine Liste von Spiel Partnern eines Spielers zurck
        //  account: Die Id des Accounts
        //  spieler: Enthlt nach erfolgreichem Aufruf eine Liste mit Account Ids der Spiel Partner
        //  karten: Enthlt nach erfolgreichem Aufruf eine Liste mit den Ids der gespielten Karten
        //  anzahl: Enthlt nach erfolgreichem Aufruf eine Liste mit der Anzahl der Spiele
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool getSpielPartnerListe( int account, Framework::Array< int > *spieler, Framework::Array< int > *karten, Framework::Array< int > *anzahl ) override;
        // Gibt eine Liste mit Spiel Statistiken zurck
        //  account: Die Id des Accounts zu dem die Statistik ermittelt werden soll
        //  seite: Die gewnschte Seite. Enthlt nach efolgreichem Aufruf die korrekte Seite
        //  maxSeite: Enthlt nach erfolgreichem Aufruf die Macimale Seite
        //  sortSpalte: Die Spalte, nach der Sortiert werden soll
        //  rckwrts: 1:=Rckwrtzt Sortierung, 0:=Vorwrts Sortierung
        //  werte: Enthlt nach erfolgreichem Aufruf eine Liste mit Statistik Werten (gespielt, gewonnen, verloren, punkte, bw0, bw1, bw2, bw3, bw4, bw5)
        //  namen: Enthlt nach erfolgreichem Aufruf die Namen der Spiele
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool getStatistikListe( int account, int *seite, int *maxSeite, char sortSpalte, char rckwrts,
                                        Framework::RCArray< Framework::Array< int > > *werte, Framework::RCArray< Framework::Text > *namen ) override;
        // Gibt eine Liste mit den Daten der letzten 20 Spiele von einem Account zurck
        //  account: Die Id des Accounts
        //  Gibt bei misserfolg 0 zurck
        Array< SpielHistorieDaten > *getSpielHistorieDaten( int account ) override;
        // Erzeugt einen Client zu dem Historie Server, welcher die Aufzeichnungen und Statistiken eines Bestimmten Spiels verwaltet
        //  spielId: Die id des Spiels
        //  Gibt bei misserfolg 0 zurck
        HistorieServerClient *createHistorieServerClient( int spielId ) override; // sucht nach dem Historie Server von einem Spiel
                                                                                     // Prft, ob der Account bestimmte Berechtigungen besitzt
                                                                                     //  recht: Die Id der Berechtigungen
        bool hatRecht( int recht ) override; // Prft ob Berechtigung vorhanden
                                                // Gibt eine Liste mit Karten zurck, die einen bestimmten Text im Namen enthalten und bestimmt sortiert ist
                                                //  filter: Die Zeichenkette, die im Namen Auftreten soll
                                                //  sortSpalte: Der Index der Spalte, nach der sortiert werden soll
                                                //  absteigend: 1:=absteigende sortierung,0:=aufsteigende Sortierung
                                                //  kId: Eine Liste, welche nach einem erfolgreichen Aufruf die Ids der Karten enthlt
                                                //  kName: Eine Liste, welche nach einem erfolgreichen Aufruf die Namen der Karten enthlt
                                                //  sName: Eine Liste, welche nach einem erfolgreichen Aufruf die Namen der Spiele der Karten enthlt
                                                //  kupfer: Eine Liste, welche nach einem erfolgreichen Aufruf die Preise der Karten in Kupfer
                                                //  verkauft: Eine Liste, welche nach einem erfolgreichen Aufruf die Anzahl der Verkufe der Karten enthlt
                                                //  maxSpieler: Eine Liste, welche nach einem erfolgreichen Aufruf die Maximalen Spieleranzahlen der Karten enthlt
        int getKartenListe( char *filter, char sortSpalte, char absteigend, Framework::Array< int > *kId, Framework::RCArray< Framework::Text > *kName, Framework::RCArray< Framework::Text > *sName,
                                    Framework::Array< int > *kupfer, Framework::Array< int > *verkauft, Framework::Array< int > *maxSpieler ) override;
        // Erzeugt einen Client zu dem Editor Server von der Karte mit einer bestimmten Id
        //  karteId: Die Id der Karte
        //  Gibt bei misserfolg 0 zurck
        EditorServerClient *createEditorServerClient( int karteId ) override;
        // Erzeugt einen Client zu dem Karten Server von der Karte mit einer bestimmten Id
        //  karteId: Die Id der Karte
        //  Gibt bei misserfolg 0 zurck
        KartenServerClient *createKartenServerClient( int karteId ) override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        InformationServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        InformationServerClient *release() override;
    };
}