#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um Minigame Bestscores abzufragen und Minigame Spielaufzeichnungen zu laden
    //  Kann nur von eingeloggten Clients verwendet werden
    class MinigameClient : public MinigameServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        MinigameClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~MinigameClient();
        // verbindet sich mit dem zugewiesenen Minigame Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // Ermittelt die liste mit allen Optionen zu einem Minigame zurck, zu denen es Welt beste Scores gibt
        //  mName: Der Name des Minigames
        //  zOptionList: Enthlt nach erfolgreichem Aufruf eine Liste mit Optionen
        //  Gibt die Anzahl der Optionen zurck
        int getMinigameOptionList( char *mName, Framework::RCArray< Framework::Text > *zOptionList ) override;
        // Ermittelt eine Liste mit den Weltbesten Scores zurck
        //  mName: Der Name des Minigames
        //  zScore: Enthlt nach erfolgreichem Aufruf eine Liste mit Scores
        //  zPlayerList: Enthlt nach erfolgreichem Aufruf eine Liste mit angezeigten Account Namen, die die Scores erreicht haben.
        //  zOptionList: Enthlt nach erfolgreichem Aufruf eine Liste mit Optionen, die beim erreichen der Scores aktiv waren.
        //  Gibt die Anzahl der Bestscores zurck
        int getMinigameBestscoreList( char *mName, Framework::Array< int > *zScore, Framework::RCArray< Framework::Text > *zPlayerList, Framework::RCArray< Framework::Text > *zOptionList ) override;
        // Gibt den Welt bestscore zu einem Bestimmten Minigame mit bestimmten Optionen zurck.
        //  mName: Der Name des Minigames
        //  oName: Die Optionen
        //  zPlayer: Enthlt nach erfolgreichem Aufruf den Angezeigten Namen des Accounts, der den Score erreicht hat
        int getMinigameOptionBestscore( char *mName, char *oName, Framework::Text *zPlayer ) override;
        // Meldet die Beendigung eines Minigames
        //  mName: Der Name des Minigames
        //  oName: Die Optionen mit denen gespielt wurde
        //  score: Der Erreichte Score
        //  zCapture: Ein Zeiger auf eine Datei mit der Spielaufzeichnung
        //  Gibt 0 zurck wenn eines Fehler aufgetreten ist, 1 wenn der Forgang erfolgreich war
        bool reportEndOfGame( char *mName, char *oName, int score, Framework::Datei *zCapture ) override;
        // Ldt ein Game Capture herunter und speichert sie unter data/tmp/minigames/wb.mgc
        //  mName: Der Name des Minigames
        //  oName: Die Optionen
        //  Gibt die Datei mit dem Capture zurck
        Framework::Datei *downloadGameCapture( char *mName, char *oName ) override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        MinigameServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        MinigameServerClient *release() override;
    };
}