#include "NewsClient.h"
#include <Klient.h>
#include "Keys.h"
#include <Globals.h>
#include <Datei.h>

using namespace KSGClient;

// Inhalt der NewsClient Klasse

// Konstruktor
NewsClient::NewsClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen )
{
    ref = 1;
    this->ip = ip;
    this->port = port;
    cId = klientId;
    k = 0;
    this->key = new char[ keyLen ];
    memcpy( this->key, key, keyLen );
    this->keyLen = keyLen;
}

// Destruktor
NewsClient::~NewsClient()
{
    trenne( 1 );
    delete[] key;
}

// verbindet ich mit dem News Server
//  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
bool NewsClient::verbinde()
{
    cs.lock();
    if( k )
    {
        cs.unlock();
        return 1;
    }
    k = new Network::Klient();
    int l = 0;
    char *key;
    Keys::getServerKey( &key, l, Keys::NEWS, Keys::SENDEN );
    k->setSendeKey( key, l );
    delete[] key;
    Keys::getServerKey( &key, l, Keys::NEWS, Keys::EMPFANGEN );
    k->setEmpfangKey( key, l );
    delete[] key;
    if( k->verbinde( port, ip ) )
    {
        if( k->sendeEncrypted( "\1", 1 ) )
        {
            k->sendeEncrypted( (char*)&cId, 4 );
            char serverReturn = 0;
            k->getNachrichtEncrypted( &serverReturn, 1 );
            if( serverReturn == 3 )
            {
                char byte = 0;
                k->getNachrichtEncrypted( &byte, 1 );
                char *f = new char[ byte + 1 ];
                f[ byte ] = 0;
                k->getNachrichtEncrypted( f, byte );
                err = "error while identifying client News Server returned: ";
                err += f;
                delete[]f;
                trenne( 0 );
                cs.unlock();
                return 0;
            }
            k->setSendeKey( this->key, this->keyLen );
            k->setEmpfangKey( this->key, this->keyLen );
        }
        else
        {
            err = "network error while sending to Patch Server";
            k = k->release();
            cs.unlock();
            return 0;
        }
    }
    else
    {
        err = "network error while connecting to Patch Server";
        k = k->release();
        cs.unlock();
        return 0;
    }
    cs.unlock();
    return 1;
}

// Ldt eine KSG-Script Seite herunter uns speichert sie unter data/tmp/news/{name}
//  name: Der Name der Seite
//  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
bool NewsClient::ladeSeite( char *name )
{
    cs.lock();
    if( !k )
    {
        err = "Der Client ist nicht verbunden.";
        cs.unlock();
        return 0;
    }
    k->sendeEncrypted( "\5", 1 );
    char ret = 0;
    k->getNachrichtEncrypted( &ret, 1 );
    if( ret == 1 )
    {
        char ln = (char)textLength( name );
        k->sendeEncrypted( &ln, 1 );
        k->sendeEncrypted( name, ln );
        k->getNachrichtEncrypted( &ret, 1 );
        if( ret == 1 )
        {
            Text *pfad = new Text( "data/tmp/news/" );
            pfad->append( name );
            if( DateiExistiert( pfad->getThis() ) )
                DateiRemove( pfad->getThis() );
            pfad->append( "/" );
            int dAnz = 0;
            k->getNachrichtEncrypted( (char*)&dAnz, 4 );
            for( int i = 0; i < dAnz; i++ )
            {
                char nLn = 0;
                k->getNachrichtEncrypted( &nLn, 1 );
                char *dName = new char[ nLn + 1 ];
                dName[ nLn ] = 0;
                k->getNachrichtEncrypted( dName, nLn );
                Text *pf = new Text( pfad->getText() );
                pf->append( dName );
                delete[] dName;
                Datei *d = new Datei();
                d->setDatei( pf );
                d->erstellen();
                d->open( Datei::Style::schreiben );
                __int64 dGr = 0;
                k->getNachrichtEncrypted( (char*)&dGr, 8 );
                char buffer[ 2048 ];
                while( dGr > 0 )
                {
                    int ln = dGr > 2048 ? 2048 : (int)dGr;
                    k->getNachricht( buffer, ln );
                    d->schreibe( buffer, ln );
                    dGr -= ln;
                }
                d->close();
                d->release();
            }
            pfad->release();
        }
    }
    if( ret == 3 )
    {
        k->getNachrichtEncrypted( &ret, 1 );
        if( ret )
        {
            char *tmp = new char[ ret ];
            k->getNachrichtEncrypted( tmp, ret );
            err = tmp;
            delete[] tmp;
        }
        else
            err = "unbekannter Fehler";
        cs.unlock();
        return 0;
    }
    cs.unlock();
    return 1;
}

// Erhlt die Verbindung aufrecht
//  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
//  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
bool NewsClient::keepAlive()
{
    if( !k )
        return 0;
    char res = 0;
    if( !cs.tryLock() )
        return 1;
    Framework::logLine( ( char* )"Verbindungstest zum News Server..." );
    bool ok = k->sendeEncrypted( "\x6", 1 );
    ok &= k->getNachrichtEncrypted( &res, 1 );
    cs.unlock();
    if( res != 1 || !ok )
    {
        Framework::logLine( ( char* )"Verbindungsabbruch." );
        trenne( 0 );
    }
    else
        Framework::logLine( ( char* )"Verbindung besteht." );
    return res == 1;
}

// Trennt die Verbindung zum Server
//  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
//  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
bool NewsClient::trenne( bool abmelden )
{
    cs.lock();
    verbinde();
    if( !k )
    {
        cs.unlock();
        return 1;
    }
    if( abmelden )
    {
        k->sendeEncrypted( "\4", 1 );
        char ret = 0;
        k->getNachrichtEncrypted( &ret, 1 );
        if( ret == 3 )
        { // error
            k->getNachrichtEncrypted( &ret, 1 );
            char *msg = new char[ ret + 1 ];
            msg[ ret ] = 0;
            if( ret )
                k->getNachrichtEncrypted( msg, ret );
            err = "error while unregister Client News Server returned: ";
            err += msg;
            delete[] msg;
        }
    }
    k->sendeEncrypted( "\3", 1 );
    char ret = 0;
    k->getNachrichtEncrypted( &ret, 1 );
    if( ret == 3 )
    { // error
        k->getNachrichtEncrypted( &ret, 1 );
        char *msg = new char[ ret + 1 ];
        msg[ ret ] = 0;
        if( ret )
            k->getNachrichtEncrypted( msg, ret );
        err = "error while trenne News Server returned: ";
        err += msg;
        delete[] msg;
    }
    k->trenne();
    k = k->release();
    cs.unlock();
    return 1;
}

// Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
bool NewsClient::istVerbunden() const
{
    return k != 0;
}

// gibt den Letzten Fehlertext zuck
//  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
char *NewsClient::getLetzterFehler() const
{
    return err;
}

// Erhht den Reference Counter um 1 un gibt this zurck
NewsServerClient *NewsClient::getThis()
{
    ref++;
    return this;
}

// Verringert den Reference Counter um 1 und gibt 0 zurck.
//  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
NewsServerClient *NewsClient::release()
{
    if( !--ref )
        delete this;
    return 0;
}