#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    class PatchClient : public PatchServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        PatchClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~PatchClient();
        // verbindet ich mit dem Patch Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // Gibt eine Tabelle von Dateigruppen zurck.
        //  Gibt bei misserfolg 0 zurck
        Framework::KSGTDatei *getDateiGruppenListe() override;
        // Gibt eine Tabelle von Dateien zurck.
        //  gruppe: Die Id der Dateigruppe, zu der die Dateien gehhren sollen
        //  Gibt bei misserfolg 0 zurck
        Framework::KSGTDatei *getDateiListe( int gruppe ) override;
        // Gibt die Gre einer Datei in bytes zurck
        //  gruppe: Die Id der Dateigruppe der Datei
        //  pfad: Der Pfad der Datei
        __int64 getDateiGre( int gruppe, const char *pfad ) override;
        // Ldt eine Datei herunter
        //  gruppe: Die Dateigruppe der Datei
        //  start: Der Index des bytes, bei dem der download begonnen werden soll (Enthlt im Falle eines Abbruchs die Position, von welcher aus als nchstes heruntergeladen werden muss)
        //  pfad: Der Pfad der Datei
        //  zielPfad: Der Pfad, an den die Datei gespeichert werden soll
        //  zFb: Ein Fortschrittsbalken, der automatisch aktualisiert wird
        //  abbruch: Wenn diese Variable whrend des Vorgangs auf 1 gesetzt wird, so wird das Herunterladen abgebrochen.
        //  maxbps: Die Anzahl der bytes pro Sekunde, die maximal bertragen werden sollen
        bool downloadDatei( int gruppe, __int64 *start, const char *pfad, const char *zielPfad, Framework::FBalken *zFb, bool *abbruch, int maxbps ) override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // Gibt die Anzahl von bertragenen Bytes seit dem letzten Aufruf zurck
        int getDownload() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        PatchServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        PatchServerClient *release() override;
    };
}