#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird benutzt, um Accounts zu erstellen zu ndern oder zu lschen
    class RegisterClient : public RegisterServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        RegisterClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~RegisterClient();
        // verbindet ich mit dem Register Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // Sendet eine Anfrage auf Erstellung eines neuen Accounts. Ein neuer Account kann erst verwendet werden, wenn ein Link in der Besttigungsemail angeklickt wurde.
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        //  eMail: Die EMail Adresse an die die besttigungs EMail gesendet werden soll
        //  jahr: Das Jahr des Geburtsdatums
        //  monat: Der Monat des Geburtstages
        //  tag: Der Tag des Geburtstages
        bool accountErstellen( const char *name, const char *pass, const char *geheim, const char *eMail, unsigned short jahr, char monat, char tag ) override;
        // Sendet eine Anfrage auf Lschung eines neuen Accounts. Ein neuer Account kann erst gelscht werden, wenn ein Link in der Besttigungsemail angeklickt wurde.
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        bool accountLschen( const char *name, const char *pass, const char *geheim ) override;
        // ndert das Passwort eines Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        //  nPass: Das neue Passwort des Accounts
        bool passwortndern( const char *name, const char *pass, const char *geheim, const char *nPass ) override;
        // ndert die EMail Adresse eines Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        //  nEMail: Die neue EMail Adresse
        bool eMailndern( const char *name, const char *pass, const char *geheim, const char *nEMail ) override;
        // ndert das Geheimnis eines Accounts
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        //  nGeheim: Das neue Geheimnis
        bool geheimnisndern( const char *name, const char *pass, const char *geheim, const char *nGeheim ) override;
        // Sendet eine Anfrage auf Name Vergessen. Der Name wird an die angegebene EMail Adresse gesendet
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        bool nameVergessen( const char *pass, const char *geheim ) override;
        // Sendet eine Anfrage auf Passwort Vergessen. Es wird ein Link an die angegebene Email Adresse gesendet, ber den ein neues Passwort vergeben werden kann
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  geheim: Das Geheimnis des Accounts
        bool passwortVergessen( const char *name, const char *geheim ) override;
        // Sendet eine Anfrage auf Geheimnis Vergessen. Das Geheimnis wird an die angegebene EMail Adresse gesendet
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        bool geheimnisVergessen( const char *name, const char *pass ) override;
        // Sendet eine Anfrage auf EMail Vergessen. Die EMail Adresse wird direkt vom Server zurckgegeben
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  name: Der Login Name des Accounts
        //  pass: Das Login Passwort des Accounts
        //  geheim: Das Geheimnis des Accounts
        //  eMail: Ein Zeiger auf eine Zeichenkette. Diese wird bei einem Erfolgreichen Aufruf auf die EMail Adresse gesetzt und muss manuell mit delete[] gelscht werden
        bool eMailVergessen( const char *name, const char *pass, const char *geheim, char **eMail ) override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        RegisterServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        RegisterServerClient *release() override;
    };
}