#pragma once

#include "../Include/KSGNetwork.h"
#include <Critical.h>

namespace KSGClient
{
    // Wird verwendet um Spiele oder Karten zu kaufen
    //  Kann nur von eingeloggten Clients verwendet werden
    class ShopClient : public ShopServerClient
    {
    private:
        int ref;
        Framework::Text ip;
        unsigned short port;
        int cId;
        Network::Klient *k;
        char *key;
        unsigned char keyLen;
        Framework::Critical cs;
        Framework::Text err;

    public:
        // Konstruktor
        ShopClient( int klientId, unsigned short port, char *ip, char *key, unsigned char keyLen );
        // Destruktor
        ~ShopClient();
        // verbindet sich mit dem zugewiesenen Shop Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool verbinde() override;
        // Gibt eine Liste mit Spiel Ids zurck, mit Spielen, die einen Bestimten Text im Namen haben
        //  suche: Der Text, der im Namen der Spiele vorkommen soll
        //  Gibt bei misserfolg 0 zurck
        Framework::Array< int > *suchSpiele( const char *suche ) override;
        // ld Titelbild des Spieles herunter und speichert es unter data/tmp/shop/kazfen/spiele/{id}/titelbg.ltdb
        //  id: Die Id des Spiels
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeSpielTitel( int id ) override;
        // ld Shop Seite des Spieles herunter und speichert es unter data/tmp/shop/kazfen/spiele/{id}/seite.ksgs
        //  id: Die Id des Spiels
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeSpielSeite( int id ) override;
        // Gibt den Besitz Status eines Spiels zurck (0 = nicht im Besitz, 1 = Testversion, 2 = gekauft)
        //  id: Die Id des Spiels
        int getSpielBesitzStatus( int id ) override;
        // gibt die Anzahl der verbleibenden Spiele der Testversion zurck
        //  id: Die Id des Spiels
        int getSpielTestversion( int id ) override;
        // Gibt den Erwerbbarkeits Status eines Spiels zurck (0 = nichts, 1 = nur Testversion, 2 = nur Vollversion, 3 = beides)
        //  id: Die Id des Spiels
        int istSpielErwerbbar( int id ) override;
        // Gibt den Preis eines Spiels in Kupfer zurck
        //  id: Die Id des Spiels
        //  testVersion: 1:=Es wird der Preis der Testversion zurckgegeben, 0:=Es wird der Preis der Vollversion zurckgegeben
        int getSpielPreis( int id, bool testVersion ) override;
        // Kauft ein Spiel
        //  id: Die Id des Spiels
        //  testVersion: 1:=Es wird die Testversion erworben, 0:=Es wird die Vollversion erworben
        bool spielErwerben( int spielId, bool testVersion ) override;
        // sucht nach Karten mit zu einem bestimmten Spiel
        //  suche: Ein Text, der im Namen der Karte vorkommen soll
        //  spielId: Die Id des Spiels
        //  Gibt eine Liste Mit den Ids der gefundenen Karten zurck
        Framework::Array< int > *suchKarten( const char *suche, int spielId ) override;
        // ldt das Titelbild der Karte herunter und speichert es unter data/tmp/shop/kaufen/karten/{id}/titelbg.ltdb
        //  id: Die Id der Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeKarteTitel( int id ) override;
        // ldt die Shop Seite der Karte herunter und speichert es unter data/tmp/shop/kaufen/karten/{id}/seite.ksgs
        //  id: Die Id der Karte
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        bool ladeKarteSeite( int id ) override;
        // Gibt den Besitz Status einer Karte zurck (0 = nicht im Besitz, 1 = Testversion, 2 = gekauft)
        //  id: Die Id der Karte
        int getKarteBesitzStatus( int id ) override;
        // gibt die Anzahl der verbleibenden Spiele der Testversion zurck
        //  id: Die Id der Karte
        int getKarteTestversion( int id ) override;
        // Gibt den Erwerbbarkeits Status einer Karte zurck (0 = nichts, 1 = nur Testversion, 2 = nur Vollversion, 3 = beides)
        //  id: Die Id der Karte
        int istKarteErwerbbar( int id ) override;
        // Gibt den Preis einer Karte in Kupfer zurck
        //  id: Die Id der Karte
        //  testVersion: 1:=Es wird der Preis der Testversion zurckgegeben, 0:=Es wird der Preis der Vollversion zurckgegeben
        int getKartePreis( int id, bool testVersion ) override;
        // Kauft eine Karte
        //  id: Die Id der Karte
        //  testVersion: 1:=Es wird die Testversion erworben, 0:=Es wird die Vollversion erworben
        bool karteErwerben( int karteId, bool testVersion ) override;
        // Erhlt die Verbindung aufrecht
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte whrend einer bestehenden Verbindung etwa einmal alle 60 Sekunden aufgerufen werden, da sonst der Router die Verbindung automatisch trennt
        bool keepAlive() override;
        // Trennt die Verbindung zum Server
        //  Gibt 1 zurck, falls der Vorgang erfolgreich ist, 0 sonnst
        //  Sollte erst nach einem erfolgreichen Aufruf von verbinde aufgerufen werden
        bool trenne( bool abmelden ) override;
        // Gibt 1 zurck, falls der Client verbunden ist, 0 sonst
        bool istVerbunden() const override;
        // gibt den Letzten Fehlertext zuck
        //  sollte erst aufgerufen werden, nachdem eine andere aufgerufene Methode fehlgeschlagen ist
        char *getLetzterFehler() const override;
        // Erhht den Reference Counter um 1 un gibt this zurck
        ShopServerClient *getThis() override;
        // Verringert den Reference Counter um 1 und gibt 0 zurck.
        //  Falls der Reference Counter nach dem Aufruf auf 0 ist lscht sich das Objekt selbst 
        ShopServerClient *release() override;
    };
}